# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import random
import urllib2
import httplib
import datetime
import re
import xml.sax
import xml.sax.saxutils
import StringIO
import HTMLParser
import htmlentitydefs
import logging

try:
    set
except NameError:
    from sets import Set as set

import phpserialize

from linkexchange.clients.base import BaseClient, SimpleFileTestServer
from linkexchange.clients.base import ClientError, \
        ClientNetworkError, ClientDataError, ClientDataAccessError
from linkexchange.utils import is_plugin_specifier, load_plugin
from linkexchange.utils import urlopen_with_timeout, get_default_user_agent

log = logging.getLogger('linkexchange.clients.sape')

class SapeTestServer(SimpleFileTestServer):
    filename = 'sape_test_server_data.txt'
    data = {
        '/': [
            '<a href="url1">link1</a>',
            '<a href="url2">link2</a>'],
        '/path/1': [
            '<a href="url1">link1</a>',
            '<a href="url2">link2</a>',
            '<a href="url3">link3</a>',
            '<a href="url4">link4</a>'],
        '/path/2': [
            'Plain text and <a href="url">link text</a>'],
        '__sape_new_url__': '<!--12345-->'}
    extra_data = None

    def __init__(self, filename = None, data = None, extra_data = None):
        if data is not None:
            self.data = data
        if extra_data is not None:
            self.extra_data = extra_data
        if self.extra_data:
            self.data.update(self.extra_data)
        raw_data = phpserialize.dumps(self.data)
        super(SapeTestServer, self).__init__(filename = filename,
                raw_data = raw_data)

class SapeClient(BaseClient):
    """
    Sape.ru client.

    >>> from linkexchange.clients.base import PageRequest
    >>> test_server = SapeTestServer()
    >>> cl = SapeClient(user = 'user123456789', db_driver = ('mem',),
    ...                 server_list = [test_server.url])
    >>> lx = cl.get_raw_links(PageRequest(url = 'http://example.com/'))
    >>> lx[0]
    u'<a href="url1">link1</a>'
    >>> lx[1]
    u'<a href="url2">link2</a>'
    >>> cl = SapeClient(user = 'user123456789', db_driver = ('mem',),
    ...                 server_list = ['file:///does_not_exists'])
    >>> cl.get_raw_links(PageRequest(url = 'http://example.com/'))
    []
    >>> cl.get_raw_links(PageRequest(url = 'http://example.com/'))
    []
    """
    def __init__(self, user, db_driver, **kw):
        """
        @param user: user ID of Sape account
        @param db_driver: multihash database driver instance or plugin specifier
        @keyword db_lifetime: DB lifetime as datetime.timedelta object or
                              number of seconds as numeric value
        @keyword db_reloadtime: if an error in DB refreshing occurred don't
                                refresh DB within this time (datetime.timedelta
                                or number of seconds)
        @keyword socket_timeout: socket timeout in seconds
        @keyword force_show_code: if True force to show check code
        @keyword server_list: list of servers URLs
        @keyword user_agent: user agent string
        """
        self.user = user
        if is_plugin_specifier(db_driver):
            db_driver = load_plugin('linkexchange.multihash_drivers', db_driver)
        self.db_driver = db_driver
        self.db_lifetime = kw.get('db_lifetime',
                datetime.timedelta(seconds = 3600))
        if type(self.db_lifetime) in (int, long, float):
            self.db_lifetime = datetime.timedelta(seconds = self.db_lifetime)
        self.db_reloadtime = kw.get('db_reloadtime',
                datetime.timedelta(seconds = 600))
        if type(self.db_reloadtime) in (int, long, float):
            self.db_reloadtime = datetime.timedelta(seconds = self.db_reloadtime)
        self.socket_timeout = kw.get('socket_timeout', 6)
        self.force_show_code = kw.get('force_show_code', True)
        self.server_list = kw.get('server_list', [
            'http://dispenser-01.sape.ru/code.php?user=%(user)s&host=%(host)s&charset=utf-8',
            'http://dispenser-02.sape.ru/code.php?user=%(user)s&host=%(host)s&charset=utf-8'])
        self.user_agent = kw.get('user_agent', get_default_user_agent())
        log.debug("New %s instance:\n%s",
                self.__class__.__name__,
                '\n'.join(["    %s: %s" % (x, repr(getattr(self, x)))
                    for x in (
                        'user',
                        'db_driver',
                        'db_lifetime',
                        'db_reloadtime',
                        'socket_timeout',
                        'force_show_code',
                        'server_list',
                        'user_agent')]))

    def _normalize_host(self, request):
        host = request.host
        if host.startswith('www.'):
            host = host[len('www.'):]
        return host

    def _load_data(self, request):
        def save_error(host, data, error):
            new_data = {}
            new_data.update(data)
            new_data['__error_time__'] =  datetime.datetime.now()
            new_data['__error_value__'] = error
            return self.db_driver.save(host, new_data, blocking = False)

        host = self._normalize_host(request)
        data = None
        force_refresh = False
        try:
            data = self.db_driver.load(host)
        except KeyError:
            log.debug("No existing database found, creating new one")
        else:
            try:
                refresh_after = data['__error_time__'] + self.db_reloadtime
            except KeyError:
                refresh_after = self.db_driver.get_mtime(host) + self.db_lifetime
            if refresh_after <= datetime.datetime.now():
                log.debug("The database too old, refreshing")
                force_refresh = True
            elif type(data.get('__sape_new_url__', None)) == str:
                # database from old version
                data = None
        if data is None:
            try:
                self.refresh_db(request)
            except ClientError, e:
                if not save_error(host, {}, e):
                    raise e
            data = self.db_driver.load(host)
        elif force_refresh:
            try:
                self.refresh_db(request)
            except ClientDataAccessError:
                pass
            except ClientError, e:
                save_error(host, data, e)
            data = self.db_driver.load(host)
        return data

    def _get_links(self, data, request):
        try:
            return data[str(request.uri)]
        except KeyError:
            if (request.cookies.get('sape_cookie') == self.user or
                    self.force_show_code):
                try:
                    return [data['__sape_new_url__']]
                except KeyError:
                    pass
        return []

    def get_raw_links(self, request):
        log.debug("Getting raw links for: %s", request.url())
        data = self._load_data(request)
        links = self._get_links(data, request)
        if request.cookies.get('sape_cookie') == self.user:
            links = ['<sape_noindex>%s</sape_noindex>' % x for x in links]
        return links

    def get_html_links(self, request):
        log.debug("Getting HTML links for: %s", request.url())
        data = self._load_data(request)
        links = self._get_links(data, request)
        delim = data.get('__sape_delimiter__', '')
        html = delim.join(links)
        if request.cookies.get('sape_cookie') == self.user:
            html = '<sape_noindex>%s</sape_noindex>' % html
        return html

    def _do_load_from_server(self, url):
        def str2uni(value):
            if type(value) == str:
                value = unicode(value, 'utf-8')
            return value
        req = urllib2.Request(url)
        req.add_header('User-Agent', self.user_agent)
        try:
            log.debug("Fetching: %s", url)
            f = urlopen_with_timeout(req, self.socket_timeout)
            raw_data = f.read()
            f.close()
        except (urllib2.URLError, httplib.HTTPException, OSError), e:
            log.error("Network error: %s", str(e))
            raise ClientNetworkError('Network error: %s' % str(e))
        if raw_data.startswith('FATAL ERROR:'):
            log.error("Server error: %s", raw_data)
            raise ClientError(raw_data)
        try:
            data = phpserialize.loads(raw_data)
        except ValueError, e:
            log.error("Could not deserialize response from server: %s", str(e))
            raise ClientDataError('Could not deserialize response '
                    'from server: %s' % str(e))
        for key, value in data.items():
            if key.startswith('/'):
                if type(value) == dict:
                    value = value.values()
                yield (key, map(str2uni, value))
            else:
                yield (key, str2uni(value))

    def refresh_db(self, request):
        host = self._normalize_host(request)
        server_list = self.server_list[:]
        random.shuffle(server_list)
        server_list = iter(server_list)
        data = None
        error = None

        while data is None:
            try:
                server = server_list.next() % dict(
                        user = self.user, host = host)
                data = self._do_load_from_server(server)
            except StopIteration:
                raise error
            except ClientError, e:
                error = e
                continue
        if not self.db_driver.save(host, data, blocking = False):
            log.warning("Other process/tread is currently writes to the database")
            raise ClientDataAccessError(
                    "Other process/tread is currently writes to the database")

class ContextLinksGenerator(HTMLParser.HTMLParser):
    def __init__(self, out, links,
            is_fragment, show_code, new_url_code,
            force_body_sape_index = False,
            exclude_tags = None,
            include_tags = None):
        HTMLParser.HTMLParser.__init__(self)
        self.out = out
        self.char_buf = []
        self.links = links
        self.is_fragment = is_fragment
        self.show_code = show_code
        self.new_url_code = new_url_code
        self.force_body_sape_index = force_body_sape_index
        self.exclude_tags = (exclude_tags or set()) | set(['a',
            'textarea', 'select', 'script', 'style',
            'label', 'noscript' , 'noindex', 'button'])
        self.exclude_ctx = []
        self.include_tags = include_tags or set()
        self.include_ctx = []

    def handle_starttag(self, tag, attrs):
        self.handle_realdata()
        if self.is_fragment:
            ignore = tag in ('html', 'body')
        else:
            ignore = not self.show_code and tag == 'sape_index'
        if not ignore:
            self.out.write(u'<' + tag)
            for k, v in attrs:
                self.out.write(' %s="%s"' % (k, xml.sax.saxutils.escape(v)))
            self.out.write(u'>')
        if tag == 'body' and self.show_code:
            if ((not self.is_fragment and self.force_body_sape_index) or
                    self.is_fragment):
                self.out.write('<sape_index>')
        if tag in self.exclude_tags:
            self.exclude_ctx.append(tag)
        elif tag in self.include_tags:
            self.include_ctx.append(tag)

    def handle_endtag(self, tag):
        self.handle_realdata()
        if tag == 'body' and self.show_code:
            if ((not self.is_fragment and self.force_body_sape_index) or
                    self.is_fragment):
                self.out.write('</sape_index>')
                if self.new_url_code:
                    self.out.write(self.new_url_code)
        if self.is_fragment:
            ignore = tag in ('html', 'body')
        else:
            ignore = not self.show_code and tag == 'sape_index'
        if not ignore:
            self.out.write('</%s>' % tag)
            if tag == 'sape_index' and self.show_code and self.new_url_code:
                self.out.write(self.new_url_code)
        if tag in self.exclude_tags:
            self.exclude_ctx.pop()
        elif tag in self.include_tags:
            self.include_ctx.pop()

    def handle_startendtag(self, tag, attrs):
        self.handle_realdata()
        self.out.write(u'<' + tag)
        for k, v in attrs:
            self.out.write(' %s="%s"' % (k, xml.sax.saxutils.escape(v)))
        self.out.write(u'/>')

    def handle_data(self, data):
        self.char_buf.append(data)

    def handle_charref(self, name):
        self.char_buf.append('&#%s;' % name)

    def handle_entityref(self, name):
        self.char_buf.append('&%s;' % name)

    def handle_realdata(self):
        content = ''.join(self.char_buf)
        self.char_buf[:] = []
        if not self.exclude_ctx:
            if set(self.include_ctx) == self.include_tags:
                for sentence_re, link_code in self.links:
                    content = sentence_re.sub(link_code, content, count = 1)
        self.out.write(content)

    def handle_comment(self, data):
        self.char_buf.append('<!--%s-->' % data)

    def handle_decl(self, data):
        self.char_buf.append('<!%s>' % data)

    def handle_pi(self, data):
        self.char_buf.append('<?%s>' % data)

class SapeContextClient(SapeClient):
    """
    Sape.ru client for context links.

    >>> from linkexchange.clients.base import PageRequest
    >>> test_server = SapeTestServer()
    >>> cl = SapeContextClient(user = 'user123456789', db_driver = ('mem',),
    ...                 server_list = [test_server.url], force_show_code = False)
    >>> html = lambda x: u'<html>\\n<body>\\n%s\\n</body>\\n</html>' % x
    >>> req = PageRequest(url = 'http://example.com/',
    ...                   cookies = dict(sape_cookie = cl.user))
    >>> print cl.content_filter(req, u'This&#x20;text contains the link1.')
    <sape_index>This&#x20;text contains the <a href="url1">link1</a>.</sape_index><!--12345-->
    >>> print cl.content_filter(req, u'foo <textarea>link1 bar</textarea>')
    <sape_index>foo <textarea>link1 bar</textarea></sape_index><!--12345-->
    >>> print cl.content_filter(req, html(u'Text link2'))
    <html>
    <body><sape_index>
    Text <a href="url2">link2</a>
    </sape_index><!--12345--></body>
    </html>
    >>> print cl.content_filter(req, html(u'<sape_index>Text link1</sape_index> &amp; Text link2.'))
    <html>
    <body>
    <sape_index>Text <a href="url1">link1</a></sape_index><!--12345--> &amp; Text link2.
    </body>
    </html>
    >>> req = PageRequest(url = 'http://example.com/',
    ...                   cookies = dict())
    >>> print cl.content_filter(req, u'This&#x20;text contains the link1.')
    This&#x20;text contains the <a href="url1">link1</a>.
    >>> print cl.content_filter(req, u'foo <textarea>link1 bar</textarea>')
    foo <textarea>link1 bar</textarea>
    >>> print cl.content_filter(req, html(u'Text link2'))
    <html>
    <body>
    Text <a href="url2">link2</a>
    </body>
    </html>
    >>> print cl.content_filter(req, html(u'<sape_index>Text link1</sape_index> &amp; Text link2.'))
    <html>
    <body>
    Text <a href="url1">link1</a> &amp; Text link2.
    </body>
    </html>
    """

    def __init__(self, user, db_driver, **kw):
        kw.setdefault('force_show_code', False)
        kw.setdefault('server_list', [
            'http://dispenser-01.sape.ru/code_context.php?user=%(user)s&host=%(host)s&charset=utf-8',
            'http://dispenser-02.sape.ru/code_context.php?user=%(user)s&host=%(host)s&charset=utf-8'])
        super(SapeContextClient, self).__init__(user, db_driver, **kw)
        self.tags_re = re.compile(r'<[^>]*>')
        self.start_doc_re = re.compile(r'^\s*<(\?xml|!DOCTYPE|html|HTML)\b', re.S)

    def get_raw_links(self, request):
        return []

    def get_html_links(self, request):
        return u''

    def content_filter(self, request, content):
        data = self._load_data(request)
        show_code = (request.cookies.get('sape_cookie') == self.user or
                self.force_show_code)
        links = data.get(str(request.uri), [])
        force_body_sape_index = False
        include_tags = set()
        if self.start_doc_re.match(content):
            is_fragment = False
            include_tags = set(['body'])
            if '<sape_index>' not in content:
                force_body_sape_index = True
            else:
                include_tags |= set(['sape_index'])
        else:
            is_fragment = True
            content = '<html><body>%s</body></html>' % content
        out = StringIO.StringIO()
        generator = ContextLinksGenerator(out, links,
                is_fragment = is_fragment,
                show_code = show_code,
                new_url_code = data.get('__sape_new_url__', ''),
                force_body_sape_index = force_body_sape_index,
                include_tags = include_tags)
        generator.feed(content)
        generator.close()
        return out.getvalue()

    def _do_load_from_server(self, url):
        def str2uni(value):
            if type(value) == str:
                value = unicode(value, 'utf-8')
            return value
        def compile(link_code):
            link_code = str2uni(link_code)
            sentence = re.escape(xml.sax.saxutils.unescape(
                self.tags_re.sub('', link_code)))
            sentence.replace(' ', r'(\s|(&nbsp;))+')
            return (re.compile(sentence, re.S + re.UNICODE), link_code)
        req = urllib2.Request(url)
        req.add_header('User-Agent', self.user_agent)
        try:
            log.debug("Fetching: %s", url)
            f = urlopen_with_timeout(req, self.socket_timeout)
            raw_data = f.read()
            f.close()
        except (urllib2.URLError, httplib.HTTPException), e:
            log.error("Network error: %s", str(e))
            raise ClientNetworkError('Network error: %s' % str(e))
        if raw_data.startswith('FATAL ERROR:'):
            log.error("Server error: %s", raw_data)
            raise ClientError(raw_data)
        try:
            data = phpserialize.loads(raw_data)
        except ValueError, e:
            log.error("Could not deserialize response from server: %s", str(e))
            raise ClientDataError('Could not deserialize response '
                    'from server: %s' % str(e))
        for key, value in data.items():
            if key.startswith('/'):
                if type(value) == dict:
                    value = value.values()
                yield (key, map(compile, value))
            else:
                yield (key, str2uni(value))

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import logging
import re

from trac.core import *
from trac.web import IRequestFilter, ITemplateStreamFilter
from genshi import XML, ParseError
from genshi.output import XMLSerializer

from linkexchange.trac import support
from linkexchange.utils import rearrange_blocks, parse_rearrange_map

log = logging.getLogger('linkexchange.trac')

class LinkExchangePlugin(Component):
    implements(IRequestFilter)
    implements(ITemplateStreamFilter)

    def __init__(self):
        super(LinkExchangePlugin, self).__init__(self)
        support.configure(self)
        self.lx_filter_template_match = re.compile(
                self.lx_options.get('filter_template_match',
                    '^wiki_view\.html$'))
    
    def pre_process_request(self, req, handler):
        return handler

    def post_process_request(self, req, template, data, content_type):
        if data is None:
            return (template, data, content_type)
        if self.lx_platform is None:
            return (template, data, content_type)

        lx_request = support.convert_request(req, self.lx_options)

	if self.lx_formatters:
            data['linkexchange_blocks'] = self.lx_platform.get_blocks(
                    lx_request, self.lx_formatters)
            try:
                rearrange_map = parse_rearrange_map(
                        self.lx_options['rearrange_map'])
            except KeyError:
                pass
            except ValueError:
                log.warning("Unable to parse rearrange_map")
            else:
                data['linkexchange_blocks'] = rearrange_blocks(page_request,
                        data['linkexchange_blocks'], rearrange_map)

        if self.lx_options.get('use_raw_links', False):
            data['linkexchange_links'] = self.lx_platform.get_raw_links(
                    lx_request)

        return (template, data, content_type)

    def filter_stream(self, req, method, filename, stream, data):
        if method != 'xhtml':
            return stream
        if self.lx_platform is None:
            return stream
        if not self.lx_options.get('content_filtering', False):
            return stream
        if not self.lx_filter_template_match.match(filename):
            return stream

        log.debug("Filtering event stream: method=%s; filename=%s",
                method, filename)

        content = ''.join(XMLSerializer()(stream))
        lx_request = support.convert_request(req, self.lx_options)
        new_content = self.lx_platform.content_filter(lx_request, content)
        try:
            stream = XML(new_content)
        except ParseError:
            log.error("Error parsing XML", exc_info = True)
            stream = XML(content)
        return stream



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import os.path
import logging

import linkexchange as lx

log = logging.getLogger('linkexchange.trac')

def configure(component):
    config = component.config
    env = component.env

    lx_log = logging.getLogger('linkexchange')
    try:
        lx_log._trac_handler
    except AttributeError:
        try:
            log_hdl = env.log._trac_handler
        except AttributeError:
            pass
        else:
            lx_log.addHandler(log_hdl)
            lx_log._trac_handler = log_hdl
            lx_log.setLevel(env.log.getEffectiveLevel())

    cfg_fn = config.getpath('linkexchange', 'config', None)
    if not cfg_fn:
        cfg_fn = os.path.join(env.path, 'conf', 'linkexchange.cfg')
        if not os.path.exists(cfg_fn):
            cfg_fn = None
    component.lx_platform = None
    component.lx_formatters = None
    component.lx_options = {}
    if cfg_fn:
        defaults = dict(
                basedir = os.path.abspath(os.path.dirname(cfg_fn)),
                envdir = os.path.abspath(env.path))
        try:
            if not lx.file_config(component.__dict__, cfg_fn,
                    defaults = defaults, prefix = 'lx_'):
                log.error("Unable to read configuration file: %s", cfg_fn)
        except lx.ConfigError, e:
            log.error("Configuration error: %s", str(e))
    else:
        log.warning("No configuration file found")

def convert_request(request, options):
    request = lx.PageRequest(
            host = options.get('host', request.environ.get('HTTP_HOST', '')),
            uri = request.href(request.path_info) + request.query_string,
            cookies = request.incookie,
            remote_addr = request.remote_addr)
    return request



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import urlparse
import Cookie
import os
import os.path

class ClientError(Exception):
    "Generic client error class"

class ClientNetworkError(ClientError):
    "Network error occurred when connecting and retrieving client data"

class ClientDataError(ClientError):
    "Client data parsing/processing error"

class ClientDataAccessError(ClientError):
    "Concurrent data access error"

class PageRequest(object):
    """
    Represents HTTP request.
    """
    def __init__(self, url = None,
            type = None, host = None, uri = None,
            cookies = None, remote_addr = None):
        if url:
            type, host, uri, query, fragment = urlparse.urlsplit(url, 'http')
            if not uri:
                uri = '/'
            if query:
                uri += '?' + query
        self.type = type or 'http'
        self.host = host
        self.uri = uri
        if isinstance(cookies, Cookie.BaseCookie):
            cookies = dict([(k, v.value) for k, v in cookies.items()])
        self.cookies = cookies or {}
        self.remote_addr = remote_addr

    def url(self):
        return '%s://%s%s' % (self.type, self.host, self.uri)


class BaseClient(object):
    """
    Base class for link exchange service clients.
    """
    def get_raw_links(self, request):
        """
        Returns list of raw HTML link codes for a given page or special tags if
        no links available.

        @param request: PageRequest object
        @return: list of unicode strings with raw HTML
        """
        return []

    def get_html_links(self, request):
        """
        Returns HTML code with links for given page. The result will be
        formatted according to service features and settings.

        @param request: PageRequest object
        @return: unicode string with HTML
        """
        return u''

    def content_filter(self, request, content):
        """
        Clients that do content filtering implements this method.

        @param request: PageRequest object
        @param content: HTML content (full page or fragment) as unicode string
        @return: filtered content as unicode string
        """
        return content

    def refresh_db(self, request):
        """
        Force to refresh client database.

        @param request: PageRequest object
        """

class SimpleFileTestServer(object):
    """
    Simple test server that stores data in file on file system and sets url
    attribute that points to it.
    """

    filename = 'test_server_data'
    raw_data = ''

    def __init__(self, filename = None, raw_data = None):
        if filename is not None:
            self.filename = filename
        if raw_data is not None:
            self.raw_data = raw_data
        open(self.filename, 'w').write(self.raw_data)
        self.url = 'file://%s' % os.path.realpath(self.filename)

    def __del__(self):
        try:
            os.unlink(self.filename)
        except OSError:
            pass




# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import shelve
import shutil
import anydbm
import datetime
import os
import select
import threading

class BaseMultiHashDriver(object):
    """
    Base multihash driver class. Multihash driver is like dictionary of
    dictionaries, it stores multiple dictionaries (hashes) accessible by key
    string. Each hash is read only, to modify use save() method passing new
    dictionary or sequence of (key, value) to it.
    """

    def load(self, key):
        """
        Loads hash associated with key. If no hash found raises KeyError.

        @param key: key string
        @return: read only dictionary like object
        """
        raise KeyError(key)

    def get_mtime(self, key):
        """
        Returns hash modification time (last save time). Raises KeyError if no
        hash found.

        @param key: key string
        @return: datetime.datetime object of last modification

        """
        raise KeyError(key)

    def save(self, key, value, blocking = True):
        """
        Creates new hash or overrides existing. New hash initialized by value
        that is dictionary or sequence of (key, value).

        If any concurrent thread/process already updates hash and blocking is
        True, then call to this method sleeps until other thread/process finish
        hash updating. If blocking is False and concurrent write access was
        detected, returns False.

        @param key: key string
        @param value: dictionary or sequence of (key, value) to initialize new hash
        @keyword blocking: use blocking or unblocking call
        @return: True if hash was saved, otherwise False
        """
        raise KeyError(key)

class MultiHashInFilesMixin:
    """
    Mixin class for multihash drivers that use files to store hash objects.
    """
    def __init__(self, filename, max_lock_time = None, no_excl = False):
        """
        @param filename: file name to use for hash files (string or callable)
        @param max_lock_time: maximum lock time in seconds or as
                              datetime.timedelta object
        @param no_excl: disable usage of O_EXCL flag when locking
        """
        self.filename = filename
        if max_lock_time is None:
            max_lock_time = datetime.timedelta(seconds = 600)
        elif type(max_lock_time) in (int, long, float):
            max_lock_time = datetime.timedelta(seconds = max_lock_time)
        self.max_lock_time = max_lock_time
        self.no_excl = no_excl

    def get_filename(self, key):
        filename = self.filename
        if callable(filename):
            filename = filename(key)
        elif 'XXX' in filename:
            filename = filename.replace('XXX', key)
        else:
            filename += key
        return filename

    def get_new_filename(self, real_filename):
        return real_filename + '.new'

    def get_lock_filename(self, real_filename):
        return real_filename + '.lock'

    def save_with_locking(self, key, value, blocking, do_save):
        real_filename = self.get_filename(key)
        lock_filename = self.get_lock_filename(real_filename)
        new_filename = self.get_new_filename(real_filename)
        error = None
        loop_cnt = 0

        while True:
            loop_cnt += 1
            if self.no_excl:
                # if usage of O_EXCL flag is disabled
                try:
                    fd = os.open(lock_filename, os.O_RDONLY)
                except OSError:
                    fd = os.open(lock_filename, os.O_CREAT)
                    break
            else:
                # locking using O_EXCL flag
                try:
                    fd = os.open(lock_filename, os.O_CREAT | os.O_EXCL)
                except OSError, e:
                    error = e
                else:
                    break
                try:
                    fd = os.open(lock_filename, os.O_RDONLY)
                except OSError:
                    if loop_cnt >= 3:
                        raise error
                    continue
            try:
                # check lock time
                lock_time = datetime.datetime.fromtimestamp(os.fstat(fd).st_ctime)
                if lock_time + self.max_lock_time <= datetime.datetime.now():
                    os.unlink(lock_filename)
                    continue
                if not blocking:
                    return False
                # wait until lock file was removed
                select.select((), (), (fd,), 5)
            finally:
                os.close(fd)
        try:
            # save data to newly created file, then just move it to the real
            # file, so that other precesses/threads can read old data while new
            # data is not completely written
            do_save(new_filename, value)
            shutil.move(new_filename, real_filename)
        finally:
            os.close(fd)
            os.unlink(lock_filename)
        return True

class MemMultiHashDriver(BaseMultiHashDriver):
    """
    Memory multihash driver.

    >>> drv = MemMultiHashDriver()
    >>> drv.save('foo', dict(bar = 3))
    True
    >>> drv.load('foo')['bar']
    3
    """
    def __init__(self):
        self.db = {}
        self.db_mtime = {}
        self.db_lock = threading.Lock()

    def load(self, key):
        return self.db[key]

    def get_mtime(self, key):
        return self.db_mtime[key]

    def save(self, key, value, blocking = True):
        result = self.db_lock.acquire(blocking)
        if not result:
            return False
        try:
            if isinstance(value, dict):
                value = value.copy()
            else:
                value = dict(value)
            self.db[key] = value
            self.db_mtime[key] = datetime.datetime.now()
        finally:
            self.db_lock.release()
        return True

class ShelveMultiHashDriver(BaseMultiHashDriver, MultiHashInFilesMixin):
    """
    Multihash driver that use shelve module to store hashes.

    >>> import os
    >>> import os.path
    >>> filename = 'shelve_multihash_test_XXX.db'
    >>> os.path.exists(filename.replace('XXX', 'foo'))
    False
    >>> drv = ShelveMultiHashDriver(filename)
    >>> drv.save('foo', dict(bar = 3))
    True
    >>> os.path.exists(filename.replace('XXX', 'foo'))
    True
    >>> drv.load('foo')['bar']
    3
    >>> def test_generator():
    ...   for i in range(100):
    ...     if i == 5:
    ...       x = drv.save('foo', dict(bar = 3), blocking = False)
    ...       assert x == False
    ...     yield ('bar%d' % i, i)
    >>> drv.save('foo', test_generator())
    True
    >>> drv.load('foo')['bar55']
    55
    >>> os.unlink(filename.replace('XXX', 'foo'))
    """
    def __init__(self, filename, max_lock_time = None, no_excl = False,
            db_module = None):
        """
        @param filename: file name to use for hash files (string or callable)
        @keyword max_lock_time: maximum lock time in seconds or as
                                datetime.timedelta object
        @keyword no_excl: disable usage of O_EXCL flag when locking
        @keyword db_module: DBM module to use
        """
        MultiHashInFilesMixin.__init__(self, filename, max_lock_time, no_excl)
        if isinstance(db_module, basestring):
            db_module = __import__(db_module)
        self.db_module = db_module

    def load(self, key):
        try:
            return shelve.open(self.get_filename(key), 'r')
        except anydbm.error:
            raise KeyError(key)

    def get_mtime(self, key):
        try:
            return datetime.datetime.fromtimestamp(os.stat(
                self.get_filename(key)).st_mtime)
        except OSError:
            raise KeyError(key)

    def save(self, key, value, blocking = True):
        def do_save(new_filename, value):
            if isinstance(value, dict):
                value = value.items()
            if self.db_module:
                # if appropriate db module is specified, use it to create empty
                # database
                db = self.db_module.open(new_filename, 'n')
                db.sync()
                db = shelve.open(new_filename, 'w')
            else:
                db = shelve.open(new_filename, 'n')
            for k, v in value:
                db[k] = v
            db.sync()
        return self.save_with_locking(key, value, blocking, do_save)

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

"""
>>> import StringIO
>>> from linkexchange import config
>>> cfg_lines = [
...   '[options]',
...   'host=example.com',
...   '',
...   '[client-1]',
...   'type=sape',
...   'user=user12345',
...   'db_driver.type=mem',
...   'server-1=http://server1.com',
...   'server-2=http://server2.com',
...   '',
...   '[client-2]',
...   'type=linkfeed',
...   'user=user12345',
...   'db_driver.type=shelve',
...   'db_driver.filename=linkfeed-XXX.db',
...   '',
...   '[formatter-1]',
...   'type=inline',
...   'count=none',
...   'delimiter=u" | "',
...   ]
>>> vars = {}
>>> cfg_file = StringIO.StringIO('\\n'.join(cfg_lines))
>>> result = config.file_config(vars, cfg_file)
>>> assert result == [cfg_file]
>>> vars['options']['host']
'example.com'
>>> vars['platform'].clients[0].user
'user12345'
>>> vars['platform'].clients[0].server_list[0]
'http://server1.com'
>>> vars['platform'].clients[0].server_list[1]
'http://server2.com'
>>> vars['platform'].clients[1].db_driver.filename
'linkfeed-XXX.db'
>>> vars['formatters'][0].count
>>> vars['formatters'][0].delimiter
u' | '
"""

import ConfigParser
import shlex

from linkexchange.utils import load_plugin
from linkexchange.platform import Platform

class ConfigError(Exception):
    "Configuration error"

def file_config(vars, fname, defaults = None, prefix = '',
        default_encoding = 'utf-8'):
    if defaults is None:
        defaults = {}

    cp = ConfigParser.ConfigParser(defaults)
    try:
        if hasattr(cp, 'readfp') and hasattr(fname, 'readline'):
            cp.readfp(fname)
            result = [fname]
        else:
            result = cp.read(fname)
    except (ConfigParser.MissingSectionHeaderError,
            ConfigParser.ParsingError), e:
        raise ConfigError("Parsing error: %s" % str(e))

    if not result:
        return result

    _clients = []
    _formatters = []
    _options = {}

    try:
        encoding = cp.get('options', 'config_encoding')
    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
        encoding = default_encoding

    for sec in cp.sections():
        opts = dict([(o, cp.get(sec, o))
            for o in cp.options(sec) if o not in defaults])
        cn = fn = None
        if sec == 'client':
            cn = 1
        elif sec.startswith('client-'):
            cn = int(sec.split('-', 1)[1])
        elif sec == 'formatter':
            fn = 1
        elif sec.startswith('formatter-'):
            fn = int(sec.split('-', 1)[1])
        elif sec == 'options':
            for k, v in opts.items():
                _options[k] = _eval_value(v, encoding)
        if cn:
            _clients.extend([None] * max(0, cn - len(_clients)))
            _clients[cn - 1] = _parse_plugin_spec(opts, encoding)
        if fn:
            _formatters.extend([None] * max(0, fn - len(_formatters)))
            _formatters[fn - 1] = _parse_plugin_spec(opts, encoding)

    def load_client(spec):
        try:
            return load_plugin('linkexchange.clients', spec)
        except ImportError:
            raise ConfigError("Client not found: %s" % spec[0])

    def load_formatter(spec):
        try:
            return load_plugin('linkexchange.formatters', spec)
        except ImportError:
            raise ConfigError("Formatter not found: %s" % spec[0])

    clients = [load_client(x) for x in _clients if x is not None]
    vars[prefix + 'platform'] = Platform(clients)
    vars[prefix + 'formatters'] = [load_formatter(x)
            for x in _formatters if x is not None]
    vars[prefix + 'options'] = _options

    return result

def _eval_value(value, encoding):
    value = value.strip()
    try:
        return int(value)
        return long(value)
        return float(value)
    except ValueError:
        pass
    value_lower = value.lower()
    if value_lower in ('true', 'on', 'enabled'):
        return True
    if value_lower in ('false', 'off', 'disabled'):
        return False
    if value_lower == 'none':
        return None
    conv_str = lambda x: x
    unquote_str = lambda x: x
    if value.endswith('"'):
        if value.startswith('u"'):
            conv_str = lambda x: unicode(x, encoding)
            value = value[1:]
        if value.startswith('"'):
            unquote_str = lambda x: ''.join(shlex.split(x))
    return conv_str(unquote_str(value))

def _parse_plugin_spec(dic, encoding):
    opts = {}
    compound = {}
    for k, v in dic.items():
        if '.' in k:
            k1, k2 = k.split('.', 1)
            compound.setdefault(k1, {})
            compound[k1][k2] = v
        elif k.endswith('_list'):
            opts[k] = [_eval_value(x, encoding) for x in v.split(',')]
        elif '-' in k and k.split('-')[-1].isdigit():
            a, n = k.split('-')
            kk = a + '_list'
            nn = int(n) - 1
            opts.setdefault(kk, [])
            opts[kk].extend([None] * max(0, nn - len(opts[kk]) + 1))
            opts[kk][nn] = _eval_value(v, encoding)
        else:
            opts[k] = _eval_value(v, encoding)
    for k, v in compound.items():
        opts[k] = _parse_plugin_spec(v, encoding)
    return (opts.pop('type'), [], opts)

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

from linkexchange.utils import is_plugin_specifier, load_plugin
from linkexchange.clients.base import ClientError, PageRequest

class Platform(object):
    """
    Platform combines various clients into one object. It have methods to get
    total list of raw links, to get links divided by blocks with custom
    formatting rules.

    >>> from linkexchange.clients.sape import SapeTestServer
    >>> srv = SapeTestServer()
    >>> clients = [('sape', [], dict(user = 'user123456789',
    ...    db_driver = ('mem',), server_list = [srv.url]))]
    >>> pl = Platform(clients = clients)
    >>> lx = pl.get_raw_links('http://example.com/')
    >>> lx[0]
    u'<a href="url1">link1</a>'
    >>> lx[1]
    u'<a href="url2">link2</a>'
    >>> formatters = [('inline', [2], dict(
    ...    class_ = 'links', class_for_empty = 'empty', suffix = '. ')),
    ...    ('list', [2], dict(id = 'links'))]
    >>> bx = pl.get_blocks('http://example.com/', formatters)
    >>> bx[0]
    u'<div class="links"><a href="url1">link1</a>. <a href="url2">link2</a>. </div>'
    >>> bx[1]
    u'<span id="links"></span>'
    >>> bx = pl.get_blocks('http://example.com/notexists', formatters)
    >>> bx[0]
    u'<div class="empty"></div><!--12345-->'
    >>> bx[1]
    u'<span id="links"></span>'
    >>> bx = pl.get_blocks('http://example.com/path/1', formatters)
    >>> bx[0]
    u'<div class="links"><a href="url1">link1</a>. <a href="url2">link2</a>. </div>'
    >>> bx[1]
    u'<ul id="links"><li><a href="url3">link3</a></li><li><a href="url4">link4</a></li></ul>'
    """

    def __init__(self, clients = None):
        """
        @param clients: list of clients instances or clients specifiers

        """
        self.clients = []
        if clients:
            for cl in clients:
                self.add_client(cl)

    def add_client(self, client):
        """
        Add link exchange service client to the platform.

        The client parameter is instance of BaseClient or client specifier.
        Client specifier is (name, args, kwargs) tuple where name is client
        name (link exchange service name), args and kwargs is list of
        positional arguments and dictionary of keyword arguments for the client
        constructor respectively.

        @param client: instance of BaseClient or client specifier
        """
        if is_plugin_specifier(client):
            client = load_plugin('linkexchange.clients', client)
        self.clients.append(client)

    def get_raw_links(self, request):
        """
        Returns list of links, just like clients returns, but catches clients
        errors and use HTML comments for errors reporting.

        @param request: PageRequest object or URL string
        @return: list of links as unicode strings
        """
        if isinstance(request, basestring):
            request = PageRequest(url = request)
        result = []
        for cl in self.clients:
            try:
                links = cl.get_raw_links(request)
            except ClientError, e:
                links = [u'<!-- %s -->' % str(e)]
            result.extend(links)
        return result

    def get_blocks(self, request, formatters):
        """
        Returns links grouped by blocks. Catches clients errors and use HTML
        comments for errors reporting.

        The request parameter is a PageRequest object or URL string. The
        formatters parameter is a sequence of BaseFormatter instances. The
        result is list of unicode strings with HTML formatted blocks of links.

        @param request: PageRequest object or URL string
        @param formatters: sequence of blocks formatters
        @return: list of links blocks as unicode strings
        """
        links = self.get_raw_links(request)
        blocks = []

        for fmt in formatters:
            if is_plugin_specifier(fmt):
                fmt2 = load_plugin('linkexchange.formatters', fmt)
            else:
                fmt2 = fmt
            tag_list = []
            link_list = []
            while links and ((fmt2.count and
                len(link_list) < fmt2.count) or (not fmt2.count)):
                x = links.pop(0)
                if '<a ' in x:
                    link_list.append(x)
                else:
                    tag_list.append(x)
            blocks.append(fmt2.format(tag_list, link_list))
        return blocks

    def content_filter(self, request, content):
        """
        @param request: PageRequest object or URL string
        @param content: HTML content (full page or fragment) as unicode string
        @return: filtered content as unicode string
        """
        if isinstance(request, basestring):
            request = PageRequest(url = request)
        for cl in self.clients:
            try:
                content = cl.content_filter(request, content)
            except ClientError, e:
                pass
        return content

    def refresh_db(self, request):
        """
        Force to refresh clients databases.

        @param request: PageRequest object or URL string
        """
        if isinstance(request, basestring):
            request = PageRequest(url = request)
        for cl in self.clients:
            try:
                cl.refresh_db(request)
            except ClientError, e:
                pass

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import pkg_resources
import urllib2
import httplib

from linkexchange import __version__ as version

def is_plugin_specifier(x):
    return type(x) in (list, tuple)

def load_plugin(space, specifier):
    specifier = list(specifier)
    name = specifier.pop(0)
    if specifier:
        args = specifier.pop(0)
    else:
        args = []
    if specifier:
        kwargs = specifier.pop(0)
    else:
        kwargs = {}
    cls = pkg_resources.load_entry_point(
            'linkexchange', space, name)
    return cls(*args, **kwargs)

def urlopen_with_timeout(url, timeout):
    class _NonBlockingConnection(httplib.HTTPConnection):
      def connect(self):
        httplib.HTTPConnection.connect(self)
        self.sock.settimeout(timeout)
     
    class _NonBlockingHandler(urllib2.HTTPHandler):
      def http_open(self, req):
        return self.do_open(_NonBlockingConnection, req)

    return urllib2.build_opener(_NonBlockingHandler).open(url)

def get_default_user_agent():
    return 'LinkExchange/%s (+http://linkexchange.org.ua)' % version

def rearrange_blocks(request, blocks, rearrange_map = None):
    """
    Rearranges links blocks according to rearrange_map and depending of request
    URI.

    >>> from linkexchange import PageRequest
    >>> request = PageRequest(host = 'example.com', uri = '/')
    >>> ord('/')
    47
    >>> blocks = [u'b1', u'b2', u'b3']
    >>> rearrange_map = [(0, 2, 0, 3), (2, 3, 3, 5)]
    >>> rearrange_blocks(request, blocks, rearrange_map)
    [u'b2', u'', u'b1', u'', u'b3']
    """
    if rearrange_map is None:
        rearrange_map = [(0, len(blocks), 0, len(blocks))]
    req_sum = sum([ord(x) for x in request.uri])
    result_dic = {}
    result_len = 0
    for i1, i2, o1, o2 in rearrange_map:
        ia = blocks[i1:i2]
        oi = o1 + (req_sum % (o2 - o1))
        while ia:
            if oi not in result_dic:
                result_dic[oi] = ia.pop(0)
            oi += 1
            if oi >= o2:
                oi = o1
        if o2 > result_len:
            result_len = o2
    return [result_dic.get(i, u"") for i in range(0, result_len)]

def parse_rearrange_map(map_str):
    """
    Parse rearrange map string as it specified in the configuration file.

    >>> parse_rearrange_map('0:1-0:3,1:2-3:5,2:3-0:3')
    [(0, 1, 0, 3), (1, 2, 3, 5), (2, 3, 0, 3)]
    """
    def parse_entry(entry):
        entry = entry.strip()
        i, o = entry.split('-')
        i1, i2 = i.split(':')
        o1, o2 = o.split(':')
        return (int(i1), int(i2), int(o1), int(o2))
    try:
        return map(parse_entry, map_str.split(','))
    except ValueError:
        raise ValueError("Invalid rearrange map string")

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import urllib2
import httplib
import random
import datetime
import xml.dom.pulldom
import logging

from linkexchange.clients.base import BaseClient, SimpleFileTestServer
from linkexchange.clients.base import ClientError, \
        ClientNetworkError, ClientDataError, ClientDataAccessError
from linkexchange.utils import is_plugin_specifier, load_plugin
from linkexchange.utils import urlopen_with_timeout, get_default_user_agent

log = logging.getLogger('linkexchange.clients.linkfeed')

class LinkFeedTestServer(SimpleFileTestServer):
    filename = 'linkfeed_test_server_data.xml'
    data = {
        '/': [
            '<a href="url1">link1</a>',
            '<a href="url2">link2</a>'],
        '/path/1': [
            '<a href="url1">link1</a>',
            '<a href="url2">link2</a>',
            '<a href="url3">link3</a>',
            '<a href="url4">link4</a>'],
        '/path/2': [
            'Plain text and <a href="url">link text</a>'],
        }
    extra_data = None

    def __init__(self, filename = None, data = None, extra_data = None):
        def make_page(url, links):
            return '<page url="%s">%s</page>' % (url,
                    ''.join(['<link><![CDATA[%s]]></link>' % s
                        for s in links]))
        if data is not None:
            self.data = data
        if extra_data is not None:
            self.extra_data = extra_data
        if self.extra_data:
            self.data.update(self.extra_data)
        raw_data = """<?xml version="1.0" encoding="UTF-8"?>
        <data>
            <config>
                <item name="end"><![CDATA[<!--12345-->]]></item>
                <item name="delimiter"><![CDATA[.]]></item>
                <item name="after_text"><![CDATA[]]></item>
                <item name="start"><![CDATA[<!--12345-->]]></item>
                <item name="before_text"><![CDATA[]]></item>
            </config>
            <pages>
            %s
            </pages>
        </data>""" % '\n'.join([make_page(url, links)
            for url, links in self.data.items()])
        super(LinkFeedTestServer, self).__init__(filename = filename,
                raw_data = raw_data)

class LinkFeedClient(BaseClient):
    """
    LinkFeed.ru client.

    >>> from linkexchange.clients.base import PageRequest
    >>> test_server = LinkFeedTestServer()
    >>> cl = LinkFeedClient(user = 'user123456789', db_driver = ('mem',),
    ...                 server_list = [test_server.url])
    >>> lx = cl.get_raw_links(PageRequest(url = 'http://example.com/'))
    >>> lx[0]
    u'<!--12345--><a href="url1">link1</a><!--12345-->'
    >>> lx[1]
    u'<!--12345--><a href="url2">link2</a><!--12345-->'
    >>> cl = LinkFeedClient(user = 'user123456789', db_driver = ('mem',),
    ...                 server_list = ['file:///does_not_exists'])
    >>> cl.get_raw_links(PageRequest(url = 'http://example.com/'))
    []
    >>> cl.get_raw_links(PageRequest(url = 'http://example.com/'))
    []
    """

    def __init__(self, user, db_driver, **kw):
        """
        @param user: user ID of LinkFeed account
        @param db_driver: multihash database driver instance or plugin specifier
        @keyword db_lifetime: DB lifetime as datetime.timedelta object or
                              number of seconds as numeric value
        @keyword db_reloadtime: if an error in DB refreshing occurred don't
                                refresh DB within this time (datetime.timedelta
                                or number of seconds)
        @keyword socket_timeout: socket timeout in seconds
        @keyword force_show_code: if True force to show check code
        @keyword server_list: list of servers URLs
        @keyword user_agent: user agent string
        """
        self.user = user
        if is_plugin_specifier(db_driver):
            db_driver = load_plugin('linkexchange.multihash_drivers', db_driver)
        self.db_driver = db_driver
        self.db_lifetime = kw.get('db_lifetime',
                datetime.timedelta(seconds = 3600))
        if type(self.db_lifetime) in (int, long, float):
            self.db_lifetime = datetime.timedelta(seconds = self.db_lifetime)
        self.db_reloadtime = kw.get('db_reloadtime',
                datetime.timedelta(seconds = 600))
        if type(self.db_reloadtime) in (int, long, float):
            self.db_reloadtime = datetime.timedelta(seconds = self.db_reloadtime)
        self.socket_timeout = kw.get('socket_timeout', 6)
        self.force_show_code = kw.get('force_show_code', True)
        self.server_list = kw.get('server_list', [
            'http://db.linkfeed.ru/%(user)s/%(host)s/UTF-8.xml'])
        self.user_agent = kw.get('user_agent', get_default_user_agent())
        log.debug("New %s instance:\n%s",
                self.__class__.__name__,
                '\n'.join(["    %s: %s" % (x, repr(getattr(self, x)))
                    for x in (
                        'user',
                        'db_driver',
                        'db_lifetime',
                        'db_reloadtime',
                        'socket_timeout',
                        'force_show_code',
                        'server_list',
                        'user_agent')]))

    def _normalize_host(self, request):
        host = request.host.lower()
        if host.startswith('www.'):
            host = host[len('www.'):]
        return host

    def _load_data(self, request):
        def save_error(host, data, error):
            new_data = {}
            new_data.update(data)
            new_data['__error_time__'] =  datetime.datetime.now()
            new_data['__error_value__'] = error
            return self.db_driver.save(host, new_data, blocking = False)

        host = self._normalize_host(request)
        data = None
        force_refresh = False
        try:
            data = self.db_driver.load(host)
        except KeyError:
            log.debug("No existing database found, creating new one")
        else:
            try:
                refresh_after = data['__error_time__'] + self.db_reloadtime
            except KeyError:
                refresh_after = self.db_driver.get_mtime(host) + self.db_lifetime
            if refresh_after <= datetime.datetime.now():
                log.debug("The database too old, refreshing")
                force_refresh = True
        if data is None:
            try:
                self.refresh_db(request)
            except ClientError, e:
                if not save_error(host, {}, e):
                    raise e
            data = self.db_driver.load(host)
        elif force_refresh:
            try:
                self.refresh_db(request)
            except ClientDataAccessError:
                pass
            except ClientError, e:
                save_error(host, data, e)
            data = self.db_driver.load(host)
        return data

    def get_raw_links(self, request):
        log.debug("Getting raw links for: %s", request.url())
        data = self._load_data(request)
        bot_ips = data.get('__linkfeed_bot_ips__', [])
        if self.force_show_code or (request.remote_addr and
                request.remote_addr in bot_ips):
            start = data.get('__linkfeed_start__', '')
            end = data.get('__linkfeed_end__', '')
        else:
            start = end = u''
        before_text = data.get('__linkfeed_before_text__', '')
        after_text = data.get('__linkfeed_after_text__', '')
        links = [start + before_text + unicode(x) + after_text + end
                for x in data.get(str(request.uri), [])]
        if not links and (start or end):
            links = [start + end]
        return links

    def get_html_links(self, request):
        log.debug("Getting HTML links for: %s", request.url())
        data = self._load_data(request)
        bot_ips = data.get('__linkfeed_bot_ips__', [])
        if self.force_show_code or (request.remote_addr and
                request.remote_addr in bot_ips):
            start = data.get('__linkfeed_start__', '')
            end = data.get('__linkfeed_end__', '')
        else:
            start = end = u''
        before_text = data.get('__linkfeed_before_text__', '')
        after_text = data.get('__linkfeed_after_text__', '')
        delimiter = data.get('__linkfeed_delimiter__', '')
        links = [unicode(x) for x in data.get(str(request.uri), [])]
        return (start + before_text + delimiter.join(links) +
                after_text + end)

    def _do_load_from_server(self, url):
        req = urllib2.Request(url)
        req.add_header('User-Agent', self.user_agent)
        try:
            log.debug("Fetching: %s", url)
            f = urlopen_with_timeout(req, self.socket_timeout)
            events = xml.dom.pulldom.parse(f)
            path = []
            node_text = lambda node: (node.firstChild is not None and
                    node.firstChild.nodeValue or u'')
            for (event, node) in events:
                if event == xml.dom.pulldom.START_ELEMENT:
                    path.append(node.tagName)
                    if path == ['data', 'pages','page']:
                        events.expandNode(node)
                        path.pop()
                        url = str(node.getAttribute('url'))
                        if url:
                            link_nodes = node.getElementsByTagName('link')
                            yield (url, [node_text(x) for x in link_nodes])
                    elif path == ['data', 'config', 'item']:
                        events.expandNode(node)
                        path.pop()
                        name = str(node.getAttribute('name'))
                        if name:
                            yield ("__linkfeed_%s__" % name,
                                    node_text(node))
                    elif path == ['data', 'bot_ips']:
                        events.expandNode(node)
                        path.pop()
                        ip_nodes = node.getElementsByTagName('ip')
                        yield ("__linkfeed_bot_ips__",
                                [node_text(x) for x in ip_nodes])
                elif event == xml.dom.pulldom.END_ELEMENT:
                    path.pop()
            f.close()
        except (urllib2.URLError, httplib.HTTPException, OSError), e:
            log.error("Network error: %s", str(e))
            raise ClientNetworkError('Network error: %s' % str(e))

    def refresh_db(self, request):
        host = self._normalize_host(request)
        server_list = self.server_list[:]
        random.shuffle(server_list)
        server_list = iter(server_list)
        data = None
        error = None

        while data is None:
            try:
                server = server_list.next() % dict(
                        user = self.user, host = host)
                data = self._do_load_from_server(server)
            except StopIteration:
                raise error
            except ClientError, e:
                error = e
                continue
        if not self.db_driver.save(host, data, blocking = False):
            log.warning("Other process/tread is currently writes to the database")
            raise ClientDataAccessError(
                    "Other process/tread is currently writes to the database")

if __name__ == "__main__":
    import doctest
    doctest.testmod()



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import logging

from linkexchange.MoinMoin import support
from linkexchange.utils import rearrange_blocks, parse_rearrange_map

log = logging.getLogger('linkexchange.MoinMoin')

Dependencies = ['time']

def execute(macro, num):
    request = macro.request
    cfg = request.cfg

    try:
	num = int(num)
    except (TypeError, ValueError):
        log.error("LinkExchangeBlock requires a single numeric argument")
	return "Required single numeric argument!"

    try:
        platform = cfg.linkexchange_platform
    except AttributeError:
        support.configure(cfg)
        platform = cfg.linkexchange_platform

    if platform is None:
	return ""

    formatters = getattr(cfg, 'linkexchange_formatters', None)
    if not formatters:
        log.error("No formatters defined for LinkExchangeBlock")
	return "No formatters defined!"

    try:
	return request.linkexchange_blocks[num]
    except AttributeError:
	pass

    page_request = support.convert_request(request)
    request.linkexchange_blocks = platform.get_blocks(
            page_request, formatters)
    try:
        rearrange_map = parse_rearrange_map(
                cfg.linkexchange_options['rearrange_map'])
    except KeyError:
        pass
    except ValueError:
        log.warning("Unable to parse rearrange_map")
    else:
        request.linkexchange_blocks = rearrange_blocks(page_request,
                request.linkexchange_blocks, rearrange_map)
    
    return request.linkexchange_blocks[num]



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import re
import logging

try:
    from MoinMoin.parser.text_moin_wiki import Parser as WikiParser
except ImportError:
    from MoinMoin.parser.wiki import Parser as WikiParser

from linkexchange.MoinMoin import support

log = logging.getLogger('linkexchange.MoinMoin')

Dependencies = ['time']

class Parser(WikiParser):
    """
    Parser to perform content filtering, e.g. calls platform.content_filter()
    on generated content.
    """

    Dependencies = Dependencies

    _fix_empty_span = re.compile(r'<span\b[^>]*></span>', re.S)

    def format(self, formatter):
        request = self.request

        try:
            platform = request.cfg.linkexchange_platform
        except AttributeError:
            support.configure(request.cfg)
            platform = request.cfg.linkexchange_platform

        if platform is None:
            return WikiParser.format(self, formatter)

        content = request.redirectedOutput(
                WikiParser.format, self, formatter)
        content = self._fix_empty_span.sub('', content)
        content = content.replace('<<<>>>', '<!--<<<>>>-->')
        try:
            content = platform.content_filter(
                    support.convert_request(request), content)
        except Exception, e:
            content += '<!-- Content filter error -->'
            log.error("Content filter error", exc_info = True)
        content = content.replace('<!--<<<>>>-->', '<<<>>>')
        request.write(content)



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import re
import Cookie
import os
import os.path
import logging

import linkexchange as lx

log = logging.getLogger('linkexchange.MoinMoin')

def configure(config):
    def check_mod_dir(mod):
        try:
            mod = __import__(mod, {}, {}, [''])
        except ImportError:
            return None
        fn = os.path.join(os.path.dirname(mod.__file__), 'linkexchange.cfg')
        if not os.path.exists(fn):
            return None
        return fn

    try:
        cfg_fn = config.linkexchange_config
    except AttributeError:
        cfg_fn = None
    if not cfg_fn:
        cfg_fn = check_mod_dir('farmconfig')
    if not cfg_fn:
        cfg_fn = check_mod_dir('wikiconfig')

    vars = dict(
            linkexchange_options = {},
            linkexchange_platform = None)
    if cfg_fn:
        defaults = dict(
                basedir = os.path.abspath(os.path.dirname(cfg_fn)))
        try:
            if not lx.file_config(vars, cfg_fn,
                    defaults = defaults, prefix = 'linkexchange_'):
                log.error("Unable to read configuration file: %s", cfg_fn)
        except lx.ConfigError, e:
            log.error("Configuration error: %s", str(e))

    for k, v in vars.items():
        if k == 'linkexchange_options':
            config.__dict__.setdefault(k, {})
            for o, ov in v.items():
                config.linkexchange_options.setdefault(o, ov)
        else:
            config.__dict__.setdefault(k, v)
    try:
        config.linkexchange_platform = lx.Platform(config.linkexchange_clients)
    except AttributeError:
        pass

    if config.linkexchange_platform is None:
        log.warning("LinkExchange is not configured")

fix_request_uri = re.compile(r'%[0-9a-f]{2}').sub

def convert_request(request):
    """
    Converts MoinMoin request object to linkexchange.PageRequest

    @param request: MoinMoin request object
    @return: linkexchange.PageRequest object
    """
    request = lx.PageRequest(
	    host = request.cfg.linkexchange_options.get(
                'host', request.http_host),
	    uri = fix_request_uri(lambda m: m.group(0).upper(),
		request.request_uri),
            cookies = Cookie.SimpleCookie(request.saved_cookie),
            remote_addr = request.remote_addr)
    return request



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import os
import os.path
import logging

from django.conf import settings
from django.contrib.sites.models import Site, RequestSite
from django.utils.http import urlquote

import linkexchange as lx

log = logging.getLogger('linkexchange.django')

platform = None
formatters = None
options = None

def configure():
    def check_mod_dir(mod):
        try:
            mod = __import__(mod, {}, {}, [''])
        except ImportError:
            return None
        fn = os.path.join(os.path.dirname(mod.__file__), 'linkexchange.cfg')
        if not os.path.exists(fn):
            return None
        return fn

    try:
        cfg_fn = settings.LINKEXCHANGE_CONFIG
    except AttributeError:
        cfg_fn = None
    if not cfg_fn:
        try:
            cfg_fn = check_mod_dir(os.environ['DJANGO_SETTINGS_MODULE'])
        except KeyError:
            pass
    if not cfg_fn:
        cfg_fn = check_mod_dir(settings.ROOT_URLCONF)

    if cfg_fn:
        defaults = dict(
                basedir = os.path.abspath(os.path.dirname(cfg_fn)))
        try:
            if not lx.file_config(globals(), cfg_fn, defaults = defaults):
                log.error("Unable to read configuration file: %s", cfg_fn)
        except lx.ConfigError, e:
            log.error("Configuration error: %s", str(e))

    global platform
    global formatters
    global options

    try:
        platform = lx.Platform(settings.LINKEXCHANGE_CLIENTS)
    except AttributeError:
        pass
    try:
        formatters = settings.LINKEXCHANGE_FORMATTERS
    except AttributeError:
        pass
    if options is None:
        options = {}
    try:
        options.update(settings.LINKEXCHANGE_OPTIONS)
    except AttributeError:
        pass

    if platform is None:
        log.warning("LinkExchange is not configured")

def convert_request(request):
    if Site._meta.installed:
        current_site = Site.objects.get_current()
    else:
        current_site = RequestSite(request)

    path = request.path
    query_string = request.environ.get('QUERY_STRING', '')
    request_uri = urlquote(path, '/:@=') + (query_string and
            ('?' + query_string) or '')

    request = lx.PageRequest(
            host = options.get('host', current_site.domain),
            uri = request_uri,
            cookies = request.COOKIES,
            remote_addr = request.META.get('REMOTE_ADDR', None))
    return request

configure()



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

__version__ = __import__('pkg_resources').get_distribution('LinkExchange').version

from linkexchange.platform import Platform
from linkexchange.clients.base import ClientError, \
        PageRequest, BaseClient
from linkexchange.config import file_config, ConfigError



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.


class BaseFormatter(object):
    def __init__(self, count):
        """
        @param count: links count in the block (0 or None - catch all)
        """
        self.count = count

    def format(self, tags, links):
        """
        Perform links formatting. tags is a list of unicode strings that don't
        contains '<a '. links is a list of unicode strings that contains links
        HTML code.

        @param tags: list of special tags
        @param links: list of links
        @return: HTML code
        """
        pass

    def _add_prefix(self, link, prefix):
        i = 0
        while not (prefix[:i] + link).startswith(prefix):
            i += 1
        link = prefix[:i] + link
        return link

    def _add_suffix(self, link, suffix):
        i = len(suffix)
        while not (link + suffix[i:]).endswith(suffix):
            i -= 1
        link += suffix[i:]
        return link

    def _format_container(self, tag, id, class_, content):
        html = u'<' + tag
        if id:
            html += u' id="%s"' % id
        if class_:
            html += u' class="%s"' % class_
        html += u'>'
        html += content
        html += u'</%s>' % tag
        return html

class InlineFormatter(BaseFormatter):
    def __init__(self, count,
            delimiter = '',
            prefix = '',
            suffix = '',
            prolog = '',
            epilog = '',
            id = None,
            class_ = None,
            class_for_empty = None):
        """
        @param count: links count in the block (None - catch all)
        @keyword delimiter: links delimiter
        @keyword prefix: links prefix
        @keyword suffix: links suffix
        @keyword prolog: text before links
        @keyword epilog: text after links
        @keyword id: value for id attribute
        @keyword class_: CSS class for nonempty block
        @keyword class_for_empty: CSS class for empty block
        """
        super(InlineFormatter, self).__init__(count)
        self.delimiter = delimiter
        self.prefix = prefix
        self.suffix = suffix
        self.prolog = prolog
        self.epilog = epilog
        self.id = id
        self.class_ = class_
        self.class_for_empty = class_for_empty

    def format(self, tags, links):
        def format_link(link):
            if self.prefix:
                link = self._add_prefix(link, self.prefix)
            if self.suffix:
                link = self._add_suffix(link, self.suffix)
            return link
        if links:
            css_class = self.class_
        else:
            css_class = self.class_for_empty
        html = self._format_container('div', self.id, css_class,
                links and (self.prolog +
                    self.delimiter.join(map(format_link, links)) +
                    self.epilog) or u'')
        html += u''.join(tags)
        return html

class ListFormatter(BaseFormatter):
    def __init__(self, count,
            prefix = '',
            suffix = '',
            id = None,
            class_ = None,
            class_for_empty = None,
            tag_for_empty = 'span'):
        """
        @param count: links count in the block (None - catch all)
        @keyword prefix: links prefix
        @keyword suffix: links suffix
        @keyword id: value for id attribute
        @keyword class_: CSS class for nonempty block
        @keyword class_for_empty: CSS class for empty block
        @keyword tag_for_empty: HTML tag for empty block
        """
        super(ListFormatter, self).__init__(count)
        self.prefix = prefix
        self.suffix = suffix
        self.id = id
        self.class_ = class_
        self.class_for_empty = class_for_empty
        self.tag_for_empty = tag_for_empty

    def format(self, tags, links):
        def format_link(link):
            if self.prefix:
                link = self._add_prefix(link, self.prefix)
            if self.suffix:
                link = self._add_suffix(link, self.suffix)
            link = u'<li>%s</li>' % link
            return link
        if links:
            css_class = self.class_
        else:
            css_class = self.class_for_empty
        html_tag = links and u'ul' or self.tag_for_empty
        html = self._format_container(html_tag, self.id, css_class,
                u''.join(map(format_link, links)))
        html += u''.join(tags)
        return html



from setuptools import setup, find_packages
setup(
    name = "LinkExchange",
    version = "0.1",
    packages = find_packages(),
    author = "Konstantin Korikov",
    author_email = "lostclus@gmail.com",
    url = "http://linkexchange.org.ua",
    download_url = "http://linkexchange.org.ua/downloads",
    description = "Universal link exchange service client library",
    long_description = """
    This library helps to integrate various link exchange services into python
    powered website. Features includes support for Sape.ru and LinkFeed.ru
    services, flexible link grouping and formatting facilities.
    """,
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Trac',
        'Framework :: TurboGears',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        ],
    license = "LGPL",
    test_suite = 'nose.collector',
    tests_require = ['nose'],
    entry_points = """
    [linkexchange.clients]
    sape = linkexchange.clients.sape:SapeClient [phpserialize]
    sape_context = linkexchange.clients.sape:SapeContextClient [phpserialize]
    linkfeed = linkexchange.clients.linkfeed:LinkFeedClient

    [linkexchange.multihash_drivers]
    mem = linkexchange.db_drivers:MemMultiHashDriver
    shelve = linkexchange.db_drivers:ShelveMultiHashDriver

    [linkexchange.formatters]
    inline = linkexchange.formatters:InlineFormatter
    list = linkexchange.formatters:ListFormatter

    [trac.plugins]
    linkexchange = linkexchange.trac.plugin
    """,
    extras_require = dict(phpserialize = "phpserialize"),
)



# LinkExchange - Universal link exchange service client
# Copyright (C) 2009 Konstantin Korikov
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# NOTE: In the context of the Python environment, I interpret "dynamic
# linking" as importing -- thus the LGPL applies to the contents of
# the modules, but make no requirements on code importing these
# modules.

import os.path
import urllib
import logging

try:
    import kid
except ImportError:
    kid = None

from turbogears import config
from cherrypy import request
import linkexchange as lx
from linkexchange.utils import rearrange_blocks, parse_rearrange_map

log = logging.getLogger('linkexchange.turbogears')

configured = False
platform = None
formatters = None
options = None

def configure():
    def check_mod_dir(mod, parent = False):
        try:
            mod = __import__(mod, {}, {}, [''])
        except (ImportError, ValueError):
            return None
        dir = os.path.dirname(mod.__file__)
        if parent:
            dir = os.path.dirname(dir)
        fn = os.path.join(dir, 'linkexchange.cfg')
        if not os.path.exists(fn):
            return None
        return fn

    cfg_fn = config.get('linkexchange.config', None)
    if not cfg_fn:
        cfg_fn = check_mod_dir(config.get('package', ''), True)

    if cfg_fn:
        defaults = dict(
                basedir = os.path.abspath(os.path.dirname(cfg_fn)))
        try:
            if not lx.file_config(globals(), cfg_fn, defaults = defaults):
                log.error("Unable to read configuration file: %s", cfg_fn)
        except lx.ConfigError, e:
            log.error("Configuration error: %s", str(e))
    else:
        log.warning("No configuration file found")

    global configured
    configured = True

def convert_request(request):
    """
    Converts cherrypy.request object to linkexchange.PageRequest object.
    """
    try:
        script_name = request.script_name
    except AttributeError:
        script_name = ''
    try:
        path = request.original_path
    except AttributeError:
        path = request.path_info
    try:
        cookies = request.cookie
    except AttributeError:
        cookies = request.simple_cookie
    try:
        remote_addr = request.remote.ip
    except AttributeError:
        remote_addr = request.remote_host

    request_uri = script_name + path
    if type(request_uri) == unicode:
        request_uri = request_uri.encode('utf-8')
    request_uri = urllib.quote(request_uri, '/:@=')
    if request.query_string:
        request_uri += '?' + request.query_string

    request = lx.PageRequest(
            host = options.get('host', request.headers['Host']),
            uri = request_uri, cookies = cookies, remote_addr = remote_addr)
    return request

def content_filter(content):
    if not configured:
        configure()

    if platform is None:
        return content

    as_kid_xml = False
    if kid is not None:
        if isinstance(content, kid.ElementStream):
            content = content.expand()
        if hasattr(content, 'tag') and hasattr(content, 'attrib'):
            content = [content]
        if type(content) == list:
            as_kid_xml = True
            orig_content = content
            s = kid.XMLSerializer(
                    namespaces = {'http://www.w3.org/1999/xhtml': ''})
            def serialize(item):
                if isinstance(item, basestring):
                    return s.escape_cdata(item)
                return unicode(s.serialize(kid.ElementStream(item),
                    encoding = 'utf-8', fragment = True), 'utf-8')
            content = u''.join(map(serialize, content))

    content = platform.content_filter(convert_request(request), content)

    if as_kid_xml:
        try:
            content = kid.XML(content).expand()
        except Exception, e:
            log.error("Error parsing XML", exc_info = True)
            return orig_content

    return content

def add_stdvars(varss):
    def as_kid_xml(value):
        try:
            return kid.XML(value).expand()
        except Exception, e:
            log.error("Error parsing XML", exc_info = True)
            return kid.XML('<!-- Error parsing XML -->')

    if not configured:
        configure()

    if platform is None:
        return

    lx_request = convert_request(request)
    vars = {}

    if formatters:
        vars['linkexchange_blocks'] = platform.get_blocks(
                lx_request, formatters)
        try:
            rearrange_map = parse_rearrange_map(options['rearrange_map'])
        except KeyError:
            pass
        except ValueError:
            log.warning("Unable to parse rearrange_map")
        else:
            vars['linkexchange_blocks'] = rearrange_blocks(lx_request,
                    vars['linkexchange_blocks'], rearrange_map)
        if options.get('as_kid_xml', False):
            vars['linkexchange_blocks'] = map(as_kid_xml,
                    vars['linkexchange_blocks'])

    if options.get('use_raw_links', False):
        vars['linkexchange_links'] = platform.get_raw_links(
                lx_request)
        if options.get('as_kid_xml', False):
            vars['linkexchange_links'] = map(as_kid_xml,
                    vars['linkexchange_links'])

    vars['linkexchange_filter'] = content_filter
    return varss.update(vars)
